* Description : The Sedici standard instruction set
*             : All instructions are 32-bit wide (2 words per instruction)
* Revision    : 0.0.2
* Author      : Mark Burrell
* Date        : 24-JUL-2009
*
*-------------------------------------------------------------------------------
Opcode        01
Mnemonic      "NOP"
Description   "No operation"

nop

*-------------------------------------------------------------------------------
Opcode        02
Mnemonic      "INTE"
Description   "psr(e) = 1"

PSR(e)=1

*-------------------------------------------------------------------------------
Opcode        03
Mnemonic      "INTD"
Description   "psr(e) = 0"

PSR(e)=0

*-------------------------------------------------------------------------------
Opcode        04
Mnemonic      "TRAP #data"
Description   "Call software interrupt mechanism"

* Programmably trigger the interrupt routine
*
* The #data value is masked with %00000111
* to produce the interrupt number
*
* Load the PSR with the Interrupt Vector
PSR(IntVec)<-[IR(operand)]
*Fire the interrupt
PSR(I)=1

*-------------------------------------------------------------------------------
Opcode        07
Mnemonic      "HALT"
Description   "Halt the processor"

halt

*-------------------------------------------------------------------------------
Opcode        08
Mnemonic      "RTI"
Description   "Return from interrupt"

ALUx<-[SSP]            *    increment
ALUr=[ALUx]+1          *    SP
SSP<-[ALUr]              *
MAR<-[SSP]               *  recover PC
MDR<-[M[MAR]]            *  from the stack
PC<-[MDR]                *
ALUx<-[SSP]            *    increment
ALUr=[ALUx]+1          *    SP
SSP<-[ALUr]              *
MAR<-[SSP]               *  recover PSR
MDR<-[M[MAR]]            *  from the stack
PSR<-[MDR]               *

*-------------------------------------------------------------------------------
Opcode        10
Mnemonic      "ADD  #data,R"
Description   "R = R + #data"

ALUx<-[IR(r0)]
ALUy<-[IR(operand)]
* Perform the addition
ALUr=[ALUx]+[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        11
Mnemonic      "ADD  addr,R"
Description   "R = R + mem[addr]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the addition
ALUr=[ALUx]+[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        12
Mnemonic      "ADD  (addr),R"
Description   "R = R + mem[(addr)]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* address in MDR
MAR<-[MDR]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the addition
ALUr=[ALUx]+[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        13
Mnemonic      "ADD  R,R"
Description   "R1 = R0 + R1"

ALUx<-[IR(r0)]
ALUy<-[IR(r1)]
* Perform the addition
ALUr=[ALUx]+[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        14
Mnemonic      "ADD  (R),R"
Description   "R1 = mem[R0] + R1"

ALUy<-[IR(r1)]
* lookup (r0)
MAR<-[IR(r0)]
MDR<-[M[MAR]]
ALUx<-[MDR]
* Perform the addition
ALUr=[ALUx]+[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        20
Mnemonic      "ADC  #data,R"
Description   "R = R + #data + psr(c)"

ALUx<-[IR(r0)]
ALUy<-[IR(operand)]
* Perform the addition
ALUr=[ALUx]+[ALUy]+[PSR(c)]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        21
Mnemonic      "ADC  addr,R"
Description   "R = R + mem[addr] + psr(c)"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the addition
ALUr=[ALUx]+[ALUy]+[PSR(c)]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        22
Mnemonic      "ADC  (addr),R"
Description   "R = R + mem[(addr)] + psr(c)"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* address in MDR
MAR<-[MDR]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the addition
ALUr=[ALUx]+[ALUy]+[PSR(c)]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        23
Mnemonic      "ADC  R,R"
Description   "R1 = R0 + R1 + psr(c)"

ALUx<-[IR(r0)]
ALUy<-[IR(r1)]
* Perform the addition
ALUr=[ALUx]+[ALUy]+[PSR(c)]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        24
Mnemonic      "ADC  (R),R"
Description   "R1 = mem[R0] + R1 + psr(c)"

ALUy<-[IR(r1)]
* lookup (r0)
MAR<-[IR(r0)]
MDR<-[M[MAR]]
ALUx<-[MDR]
* Perform the addition
ALUr=[ALUx]+[ALUy]+[PSR(c)]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        30
Mnemonic      "SUB  #data,R"
Description   "R = R - #data"

ALUx<-[IR(r0)]
ALUy<-[IR(operand)]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        31
Mnemonic      "SUB  addr,R"
Description   "R = R - mem[addr]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        32
Mnemonic      "SUB  (addr),R"
Description   "R = R - mem[(addr)]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* address in MDR
MAR<-[MDR]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        33
Mnemonic      "SUB  R,R"
Description   "R1 = R1 - R0"

ALUy<-[IR(r0)]
ALUx<-[IR(r1)]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        34
Mnemonic      "SUB  (R),R"
Description   "R1 = R1 - mem[R0]"

ALUx<-[IR(r1)]
* lookup (r0)
MAR<-[IR(r0)]
MDR<-[M[MAR]]
ALUy<-[MDR]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        40
Mnemonic      "AND  #data,R"
Description   "R = R & #data"

ALUx<-[IR(r0)]
ALUy<-[IR(operand)]
* Perform the and
ALUr=[ALUx]&[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        41
Mnemonic      "AND  addr,R"
Description   "R = R & mem[addr]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the and
ALUr=[ALUx]&[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        42
Mnemonic      "AND  (addr),R"
Description   "R = R & mem[(addr)]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* address in MDR
MAR<-[MDR]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the and
ALUr=[ALUx]&[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        43
Mnemonic      "AND  R,R"
Description   "R1 = R0 & R1"

ALUx<-[IR(r0)]
ALUy<-[IR(r1)]
* Perform the and
ALUr=[ALUx]&[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        44
Mnemonic      "AND  (R),R"
Description   "R1 = R1 & (R0)"

ALUx<-[IR(r1)]
* lookup (r0)
MAR<-[IR(r0)]
MDR<-[M[MAR]]
ALUy<-[MDR]
* Perform the and
ALUr=[ALUx]&[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        50
Mnemonic      "OR   #data,R"
Description   "R = R | #data"

ALUx<-[IR(r0)]
ALUy<-[IR(operand)]
* Perform the or
ALUr=[ALUx]|[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        51
Mnemonic      "OR   addr,R"
Description   "R = R | mem[addr]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the or
ALUr=[ALUx]|[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        52
Mnemonic      "OR   (addr),R"
Description   "R = R | mem[(addr)]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* address in MDR
MAR<-[MDR]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the or
ALUr=[ALUx]|[ALUy]
* Store the result in R
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        53
Mnemonic      "OR   R,R"
Description   "R1 = R0 | R1"

ALUx<-[IR(r0)]
ALUy<-[IR(r1)]
* Perform the and
ALUr=[ALUx]|[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        54
Mnemonic      "OR   (R),R"
Description   "R1 = R1 | (R0)"

ALUx<-[IR(r1)]
* lookup (r0)
MAR<-[IR(r0)]
MDR<-[M[MAR]]
ALUy<-[MDR]
* Perform the or
ALUr=[ALUx]|[ALUy]
* Store the result in R
IR(r1)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        60
Mnemonic      "CMP  #data,R"
Description   "R - #data"

ALUx<-[IR(r0)]
ALUy<-[IR(operand)]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]
* We don't store the result

*-------------------------------------------------------------------------------
Opcode        61
Mnemonic      "CMP  addr,R"
Description   "R - mem[addr]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]

*-------------------------------------------------------------------------------
Opcode        62
Mnemonic      "CMP  (addr),R"
Description   "R - mem[(addr)]"

ALUx<-[IR(r0)]
MAR<-[IR(operand)]
MDR<-[M[MAR]]
* address in MDR
MAR<-[MDR]
MDR<-[M[MAR]]
* place value in ALUy
ALUy<-[MDR]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]

*-------------------------------------------------------------------------------
Opcode        63
Mnemonic      "CMP  R,R"
Description   "R1 - R0"

ALUx<-[IR(r1)]
ALUy<-[IR(r0)]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]

*-------------------------------------------------------------------------------
Opcode        64
Mnemonic      "CMP  (R),R"
Description   "R1 - mem[R0]"

ALUx<-[IR(r1)]
* lookup (r0)
MAR<-[IR(r0)]
MDR<-[M[MAR]]
ALUy<-[MDR]
* Perform the subtraction
ALUr=[ALUx]-[ALUy]

*-------------------------------------------------------------------------------
Opcode        70
Mnemonic      "ROL  addr"
Description   "mem[addr] = mem[addr] <<= 1"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<=1
MDR<-[ALUr]    
M[MAR]<-[MDR]

*-------------------------------------------------------------------------------
Opcode        71
Mnemonic      "ROL  (addr)"
Description   "mem[(addr)] = mem[(addr)] <<= 1"

MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<=1    
MDR<-[ALUr]    
M[MAR]<-[MDR]

*-------------------------------------------------------------------------------
Opcode        72
Mnemonic      "ROL  R"
Description   "R = R <<= 1"

ALUx<-[ir(r0)]    
ALUr=[ALUx]<<=1    
ir(r0)<-[ALUr] 

*-------------------------------------------------------------------------------
Opcode        73
Mnemonic      "ROL  (R)"
Description   "(R) = (R) <<= 1"

MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<=1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        77
Mnemonic      "ROR  addr"
Description   "mem[addr] = mem[addr] =>> 1"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]=>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        78
Mnemonic      "ROR  (addr)"
Description   "mem[(addr)] = mem[(addr)] =>> 1"

MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]=>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        79
Mnemonic      "ROR  R"
Description   "R = R =>> 1"

ALUx<-[ir(r0)]    
ALUr=[ALUx]=>>1    
ir(r0)<-[ALUr] 

*-------------------------------------------------------------------------------
Opcode        7A
Mnemonic      "ROR  (R)"
Description   "(R) = (R) =>> 1"

MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]=>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        80
Mnemonic      "ASL  addr"
Description   "mem[addr] = mem[addr] << 1"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        81
Mnemonic      "ASL  (addr)"
Description   "mem[(addr)] = mem[(addr)] << 1"

MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<1    
MDR<-[ALUr]    
M[MAR]<-[MDR]

*-------------------------------------------------------------------------------
Opcode        82
Mnemonic      "ASL  R"
Description   "R = R << 1"

ALUx<-[ir(r0)]    
ALUr=[ALUx]<<1    
ir(r0)<-[ALUr] 

*-------------------------------------------------------------------------------
Opcode        83
Mnemonic      "ASL  (R)"
Description   "(R) = (R) << 1"

MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        87
Mnemonic      "ASR  addr"
Description   "mem[addr] = mem[addr] >> 1"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        88
Mnemonic      "ASR  (addr)"
Description   "mem[(addr)] = mem[(addr)] >> 1"

MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        89
Mnemonic      "ASR  R"
Description   "R = R >> 1"

ALUx<-[ir(r0)]    
ALUr=[ALUx]>>1    
ir(r0)<-[ALUr] 

*-------------------------------------------------------------------------------
Opcode        8A
Mnemonic      "ASR  (R)"
Description   "(R) = (R) >> 1"

MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        90
Mnemonic      "LSL  addr"
Description   "mem[addr] = mem[addr] <<< 1"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<<1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        91
Mnemonic      "LSL  (addr)"
Description   "mem[(addr)] = mem[(addr)] <<< 1"

MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<<1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        92
Mnemonic      "LSL  R"
Description   "R = R <<< 1"

ALUx<-[ir(r0)]    
ALUr=[ALUx]<<<1    
ir(r0)<-[ALUr]  

*-------------------------------------------------------------------------------
Opcode        93
Mnemonic      "LSL  (R)"
Description   "(R) = (R) <<< 1"

MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]<<<1    
MDR<-[ALUr]    
M[MAR]<-[MDR]    

*-------------------------------------------------------------------------------
Opcode        97
Mnemonic      "LSR  addr"
Description   "mem[addr] = mem[addr] >>> 1"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]>>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        98
Mnemonic      "LSR (addr)"
Description   "mem[(addr)] = mem[(addr)] >>> 1"

MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]>>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        99
Mnemonic      "LSR  R"
Description   "R = R >>> 1"

ALUx<-[ir(r0)]    
ALUr=[ALUx]>>>1    
ir(r0)<-[ALUr]  

*-------------------------------------------------------------------------------
Opcode        9A
Mnemonic      "LSR  (R)"
Description   "(R) = (R) >>> 1"

MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=[ALUx]>>>1    
MDR<-[ALUr]    
M[MAR]<-[MDR]    

*-------------------------------------------------------------------------------
Opcode        A0
Mnemonic      "NOT  addr"
Description   "mem[addr]= ~mem[addr]"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=~[ALUx]    
MDR<-[ALUr]    
M[MAR]<-[MDR]   

*-------------------------------------------------------------------------------
Opcode        A1
Mnemonic      "NOT  (addr)"
Description   "mem[(addr)] = ~mem[(addr)]"

MAR<-[IR(operand)]   
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=~[ALUx]    
MDR<-[ALUr]    
M[MAR]<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        A2
Mnemonic      "NOT  R"
Description   "R = ~R"

ALUx<-[IR(r0)]  
ALUr=~[ALUx]    
IR(r0)<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        A3
Mnemonic      "NOT  (R)"
Description   "(R) = ~(R)"

MAR<-[IR(r0)] 
MDR<-[M[MAR]]    
ALUx<-[MDR]    
ALUr=~[ALUx]    
MDR<-[ALUr]    
M[MAR]<-[MDR]    

*-------------------------------------------------------------------------------
Opcode        B0
Mnemonic      "JMP  addr"
Description   "PC = mem[addr]"

PC<-[IR(operand)]

*-------------------------------------------------------------------------------
Opcode        B1
Mnemonic      "JMP  (addr)"
Description   "PC = mem[(addr)]"

MAR<-[IR(operand)]
MDR<-[M[MAR]]
PC<-[MDR]

*-------------------------------------------------------------------------------
Opcode        B2
Mnemonic      "JMP  R"
Description   "PC = R"

PC<-[IR(r0)]  

*-------------------------------------------------------------------------------
Opcode        B3
Mnemonic      "JMP  (R)"
Description   "PC = (R)"

MAR<-[IR(r0)] 
MDR<-[M[MAR]]
PC<-[MDR]

*-------------------------------------------------------------------------------
Opcode        B4
Mnemonic      "JMP  #dis"
Description   "PC = PC + displacement"

ALUx<-[PC]    
ALUy<-[IR(operand)]
* don't change PSR
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]    
* recover PSR
PSR<-[MDR]
PC<-[ALUr] 

*-------------------------------------------------------------------------------
Opcode        C0
Mnemonic      "JSR  addr"
Description   "Jump to subroutine at mem[addr]"

MDR<-[PC]              *     write PC
MAR<-[SSP]             *     to the stack
M[MAR]<-[MDR]          *
MDR<-[PSR]
ALUx<-[SSP]               *  decrement
ALUr=[ALUx]-1             *  SP
SSP<-[ALUr]               *
PSR<-[MDR]
PC<-[IR(operand)]

*-------------------------------------------------------------------------------
Opcode        C1
Mnemonic      "JSR  (addr)"
Description   "Jump to subroutine at mem[(addr)]"

MDR<-[PC]              *     write PC
MAR<-[SSP]             *     to the stack
M[MAR]<-[MDR]          *
MDR<-[PSR]
ALUx<-[SSP]               *  decrement
ALUr=[ALUx]-1             *  SP
SSP<-[ALUr]               *
PSR<-[MDR]
MAR<-[IR(operand)]
MDR<-[M[MAR]]    
PC<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        C2
Mnemonic      "JSR  R"
Description   "Jump to subroutine at R"

ALUx<-[PC]              *
MDR<-[ALUx]             *     write PC
MAR<-[SSP]              *     to the stack
M[MAR]<-[MDR]           *
MDR<-[PSR]
ALUx<-[SSP]                *  decrement
ALUr=[ALUx]-1              *  SP
SSP<-[ALUr]                *
PSR<-[MDR]
PC<-[ir(r0)]   

*-------------------------------------------------------------------------------
Opcode        C3
Mnemonic      "JSR  (R)"
Description   "Jump to subroutine at (R)"

ALUx<-[PC]             *
MDR<-[ALUx]            *     write PC
MAR<-[SSP]             *     to the stack
M[MAR]<-[MDR]          *
MDR<-[PSR]
ALUx<-[SSP]               *  decrement
ALUr=[ALUx]-1             *  SP
SSP<-[ALUr]               *
PSR<-[MDR]
MAR<-[ir(r0)]    
MDR<-[M[MAR]]    
PC<-[MDR]   

*-------------------------------------------------------------------------------
Opcode        C4
Mnemonic      "JSR  #dis"
Description   "Jump to subroutine at PC + displacement"

ALUx<-[PC]             *
MDR<-[ALUx]            *     write PC
MAR<-[SSP]             *     to the stack
M[MAR]<-[MDR]          *
MDR<-[PSR]
ALUx<-[SSP]               *  decrement
ALUr=[ALUx]-1             *  SP
SSP<-[ALUr]               *
ALUx<-[PC]
ALUy<-[IR(operand)]
ALUr=[ALUx]+[ALUy]    
PC<-[ALUr]   
PSR<-[MDR]

*-------------------------------------------------------------------------------
Opcode        C7
Mnemonic      "RTS"
Description   "Return from subroutine"

MDR<-[PSR]          * save the PSR so its not lost
ALUx<-[SSP]            *     increment
ALUr=[ALUx]+1          *     SP
SSP<-[ALUr]            *
PSR<-[MDR]          * recover the PSR
MAR<-[SSP]                *  recover PC
MDR<-[M[MAR]]             *  from the stack
PC<-[MDR]                 *

*-------------------------------------------------------------------------------
Opcode        D0
Mnemonic      "BCC  #dis"
Description   "if psr(c)==0 then PC = PC + displacement"

if(PSR(c)==0)
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        D1
Mnemonic      "BCC  addr"
Description   "if psr(c)==0 then PC = mem[addr]"

if(PSR(c)==0)    
PC<-[IR(operand)]  

*-------------------------------------------------------------------------------
Opcode        D2
Mnemonic      "BCC  (addr)"
Description   "if psr(c)==0 then PC = mem[(addr)]"

if(PSR(c)==0)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
PC<-[MDR]   

*-------------------------------------------------------------------------------
Opcode        D3
Mnemonic      "BPL  #dis"
Description   "if psr(n)==0 then PC = PC + displacement"

if(PSR(n)==0)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        D4
Mnemonic      "BPL  addr"
Description   "if psr(n)==0 then PC = mem[addr]"

if(PSR(n)==0)    
PC<-[IR(operand)] 

*-------------------------------------------------------------------------------
Opcode        D5
Mnemonic      "BPL  (addr)"
Description   "if psr(n)==0 then PC = mem[(addr)]"

if(PSR(n)==0)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
PC<-[MDR]

*-------------------------------------------------------------------------------
Opcode        D6
Mnemonic      "BNE  #dis"
Description   "if psr(z)==0 then PC = PC + displacement"

if(PSR(z)==0)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        D7
Mnemonic      "BNE  addr"
Description   "if psr(z)==0 then PC = mem[addr]"

if(PSR(z)==0)    
PC<-[IR(operand)] 

*-------------------------------------------------------------------------------
Opcode        D8
Mnemonic      "BNE  (addr)"
Description   "if psr(z)==0 then PC = mem[(addr)]"

if(PSR(z)==0)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]       
MAR<-[MDR]
MDR<-[M[MAR]]       
PC<-[MDR]

*-------------------------------------------------------------------------------
Opcode        D9
Mnemonic      "BVC  #dis"
Description   "if psr(v)==0 then PC = PC + displacement"

if(PSR(v)==0)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        DA
Mnemonic      "BVC  addr"
Description   "if psr(v)==0 then PC = mem[addr]"

if(PSR(v)==0)    
PC<-[IR(operand)] 

*-------------------------------------------------------------------------------
Opcode        DB
Mnemonic      "BVC  (addr)"
Description   "if psr(v)==0 then PC = mem[(addr)]"

if(PSR(v)==0)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
PC<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        E0
Mnemonic      "BCS  #dis"
Description   "if psr(c)==1 then PC = PC + displacement"

if(PSR(c)==1)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        E1
Mnemonic      "BCS  addr"
Description   "if psr(c)==1 then PC = mem[addr]"

if(PSR(c)==1)    
PC<-[IR(operand)]   

*-------------------------------------------------------------------------------
Opcode        E2
Mnemonic      "BCS  (addr)"
Description   "if psr(c)==1 then PC = mem[(addr)]"

if(PSR(c)==1)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
PC<-[MDR]    

*-------------------------------------------------------------------------------
Opcode        E3
Mnemonic      "BMI  #dis"
Description   "if psr(n)==1 then PC = PC + displacement"

if(PSR(n)==1)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        E4
Mnemonic      "BMI  addr"
Description   "if psr(n)==1 then PC = mem[addr]"

if(PSR(n)==1)    
PC<-[IR(operand)] 

*-------------------------------------------------------------------------------
Opcode        E5
Mnemonic      "BMI  (addr)"
Description   "if psr(n)==1 then PC = mem[(addr)]"

if(PSR(n)==1)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]] 
PC<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        E6
Mnemonic      "BEQ  #dis"
Description   "if psr(z)==1 then PC = PC + displacement"

if(PSR(z)==1)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        E7
Mnemonic      "BEQ  addr"
Description   "if psr(z)==1 then PC = mem[addr]"

if(PSR(z)==1)    
PC<-[IR(operand)]

*-------------------------------------------------------------------------------
Opcode        E8
Mnemonic      "BEQ  (addr)"
Description   "if psr(z)==1 then PC = mem[(addr)]"

if(PSR(z)==1)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]       
MAR<-[MDR]
MDR<-[M[MAR]]       
PC<-[MDR]

*-------------------------------------------------------------------------------
Opcode        E9
Mnemonic      "BVS  #dis"
Description   "if psr(v)==1 then PC = PC + displacement"

if(PSR(v)==1)    
ALUx<-[PC]
ALUy<-[IR(operand)]
MDR<-[PSR]
ALUr=[ALUx]+[ALUy]
PSR<-[MDR]
PC<-[ALUr]

*-------------------------------------------------------------------------------
Opcode        EA
Mnemonic      "BVS  addr"
Description   "if psr(v)==1 then PC = mem[addr]"

if(PSR(v)==1)    
PC<-[IR(operand)]  

*-------------------------------------------------------------------------------
Opcode        EB
Mnemonic      "BVS  (addr)"
Description   "if psr(v)==1 then PC = mem[(addr)]"

if(PSR(v)==1)    
MAR<-[IR(operand)]    
MDR<-[M[MAR]]    
MAR<-[MDR]    
MDR<-[M[MAR]]    
PC<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        F0
Mnemonic      "MOVE #data,R"
Description   "R = data"

IR(r0)<-[IR(operand)] 

*-------------------------------------------------------------------------------
Opcode        F1
Mnemonic      "MOVE addr,R"
Description   "R = mem[addr]"

MAR<-[IR(operand)]
MDR<-[M[MAR]]
IR(r0)<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        F2
Mnemonic      "MOVE (addr),R"
Description   "R = mem[(addr)]"

MAR<-[IR(operand)]
MDR<-[M[MAR]]    
MAR<-[MDR]
MDR<-[M[MAR]]    
IR(r0)<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        F3
Mnemonic      "MOVE R,R"
Description   "R1 = R0"

ir(r1)<-[ir(r0)]

*-------------------------------------------------------------------------------
Opcode        F4
Mnemonic      "MOVE (R),R"
Description   "R1 = mem[R0]"

MAR<-[IR(r0)]
MDR<-[M[MAR]]    
IR(r1)<-[MDR] 

*-------------------------------------------------------------------------------
Opcode        F5
Mnemonic      "MOVE R,addr"
Description   "mem[addr] = R"

MAR<-[IR(operand)]
MDR<-[IR(r0)]
M[MAR]<-[MDR]

*-------------------------------------------------------------------------------
Opcode        F6
Mnemonic      "MOVE R,(addr)"
Description   "mem[(addr)] = R"

MAR<-[IR(operand)]
MDR<-[M[MAR]]   
MAR<-[MDR]
MDR<-[IR(r0)]
M[MAR]<-[MDR]

*-------------------------------------------------------------------------------
Opcode        F7
Mnemonic      "MOVE R,(R)"
Description   "mem[R1] = R0"

MAR<-[ir(r1)]
MDR<-[ir(r0)]
M[MAR]<-[MDR]

*-------------------------------------------------------------------------------
Opcode        F8
Mnemonic      "PUSH R"
Description   "Push R onto the user stack"

* Place the stack pointer in the MAR
MAR<-[USP]
* Place the register contents in the MDR
MDR<-[IR(r0)]
* Perform a memory write operation
M[MAR]<-[MDR]  
MDR<-[PSR]
* Update the stack pointer  
ALUx<-[USP]
ALUr=[ALUx]-1    
USP<-[ALUr]
PSR<-[MDR]

*-------------------------------------------------------------------------------
Opcode        F9
Mnemonic      "POP  R"
Description   "Pop R from the user stack"

MDR<-[PSR]
ALUx<-[USP]    
ALUr=[ALUx]+1    
PSR<-[MDR]
USP<-[ALUr]    
MAR<-[USP]    
MDR<-[M[MAR]]    
ir(r0)<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        FA
Mnemonic      "PSHS R"
Description   "Push R onto the supervisor stack"

* Place the stack pointer in the MAR
MAR<-[SSP]
* Place the register contents in the MDR
MDR<-[IR(r0)]
* Perform a memory write operation
M[MAR]<-[MDR] 
MDR<-[PSR] 
* Update the stack pointer  
ALUx<-[SSP]
ALUr=[ALUx]-1    
SSP<-[ALUr]
PSR<-[MDR]

*-------------------------------------------------------------------------------
Opcode        FB
Mnemonic      "POPS R"
Description   "Pop R from the supervisor stack"

MDR<-[PSR]
ALUx<-[SSP]    
ALUr=[ALUx]+1    
PSR<-[MDR]
SSP<-[ALUr]    
MAR<-[SSP]    
MDR<-[M[MAR]]    
ir(r0)<-[MDR]  

*-------------------------------------------------------------------------------
Opcode        fetch
Mnemonic      "N/A"
Description   "Standard fetch cycle"

* Lookup memory at the address held in PC
MAR<-[PC]
INC<-[PC]
PC<-[INC]
MDR<-[M[MAR]]
*
* Place the 16-bit value found into the IR
IR(cmd)<-[MDR]
*
* place data word into IR(operand) for 
* later use (if required)
MAR<-[PC]
INC<-[PC]
PC<-[INC]
MDR<-[M[MAR]]
IR(operand)<-[MDR]
* Pass the IR(opcode) to the CU for decoding
CU<-[IR(opcode)]

*-------------------------------------------------------------------------------
Opcode        interrupt
Mnemonic      "N/A"
Description   "Standard interrupt handler"

* interrupt flag = 0
PSR(I)=0
* save the PSR on the stack
MAR<-[SSP]
MDR<-[PSR]
M[MAR]<-[MDR]
* decrement the SP
ALUx<-[SSP]
ALUr=[ALUx]-1
SSP<-[ALUr]
* write PC to the stack
ALUx<-[PC]
MDR<-[ALUx]
MAR<-[SSP]
M[MAR]<-[MDR]
* Decrement SP
ALUx<-[SSP]
ALUr=[ALUx]-1
SSP<-[ALUr]
* interrupt enable flag = 0
PSR(E)=0
* Build the vector address
ALUy<-[JUMPERS(IntBase)]
ALUx<-[PSR(IntVec)]
ALUr=[ALUx]+[ALUy]
* Obtain the handler address
MAR<-[ALUr]
MDR<-[M[MAR]]
* Load the address of the interrupt handler into PC
PC<-[MDR]

